"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MonacoTreeSitter = void 0;
const lodashDebounce = require("lodash.debounce");
const highlighter_1 = require("./highlighter");
const theme_1 = require("./theme");
const monaco_1 = require("./monaco");
__exportStar(require("./theme"), exports);
__exportStar(require("./language"), exports);
__exportStar(require("./highlighter"), exports);
__exportStar(require("./highlight"), exports);
function monacoPositionToParserPoint(position) {
    return { row: position.lineNumber, column: position.column };
}
class MonacoTreeSitter {
    constructor(Monaco, editor, language, debounceUpdate = 15) {
        this.editor = editor;
        this.language = language;
        this.monacoDecorationKeys = [];
        monaco_1.provideMonacoModule(Monaco);
        this.tree = language ? language.parser.parse(editor.getValue()) : null;
        this.buildHighlightDebounced =
            debounceUpdate == null ? this.buildHighlight : lodashDebounce(this.buildHighlight.bind(this), debounceUpdate);
        const eventListener = editor.getModel().onDidChangeContent(this.onEditorContentChange.bind(this));
        this.dispose = () => {
            eventListener.dispose();
            this.language = this.tree = null;
            this.buildHighlight();
        };
        this.buildHighlight();
    }
    onEditorContentChange(e) {
        if (!this.language)
            return;
        if (e.changes.length == 0)
            return;
        for (const change of e.changes) {
            const startIndex = change.rangeOffset;
            const oldEndIndex = change.rangeOffset + change.rangeLength;
            const newEndIndex = change.rangeOffset + change.text.length;
            const startPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(startIndex));
            const oldEndPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(oldEndIndex));
            const newEndPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(newEndIndex));
            this.tree.edit({ startIndex, oldEndIndex, newEndIndex, startPosition, oldEndPosition, newEndPosition });
        }
        this.tree = this.language.parser.parse(this.editor.getValue(), this.tree);
        this.buildHighlightDebounced();
    }
    buildHighlight() {
        const decorations = this.language ? highlighter_1.buildDecorations(this.tree, this.language) : null;
        const monacoDecorations = [];
        if (decorations)
            for (const [term, ranges] of Object.entries(decorations)) {
                const options = {
                    inlineClassName: theme_1.Theme.getClassNameOfTerm(term)
                };
                for (const range of ranges) {
                    monacoDecorations.push({ range, options });
                }
            }
        this.monacoDecorationKeys = this.editor.deltaDecorations(this.monacoDecorationKeys, monacoDecorations);
    }
    changeLanguage(language) {
        this.language = language;
        this.tree = language ? language.parser.parse(this.editor.getValue()) : null;
        this.buildHighlight();
    }
    refresh() {
        this.buildHighlight();
    }
}
exports.MonacoTreeSitter = MonacoTreeSitter;
//# sourceMappingURL=index.js.map